@@ -101,6 +101,11 @@ def ppdMakeModelSplit (ppd_make_and_model):
           l.startswith ("pagepro")):
         make = "KONICA MINOLTA"
         model = ppd_make_and_model
+    elif (l.startswith ("fs-") or
+          l.startswith ("km-") or
+          l.startswith ("taskalfa")):
+        make = "Kyocera Mita"
+        model = ppd_make_and_model
     elif l.startswith ("aficio"):
         make = "Ricoh"
         model = ppd_make_and_model
@@ -194,6 +199,9 @@ def ppdMakeModelSplit (ppd_make_and_model):
                    " zxs",              # hpcups
                    " pcl3",             # hpcups
                    "_bt",
+                   " pcl",              # Canon CQue
+                   " ufr ii",           # Canon UFR II
+                   " br-script3",       # Brother PPDs
                    ","]:
         s = modell.find (suffix)
         if s != -1:
@@ -225,13 +233,13 @@ DRIVER_TYPE_DOWNLOADED_NOW = 5
 DRIVER_TYPE_FOOMATIC_RECOMMENDED_NON_POSTSCRIPT = 8
 DRIVER_TYPE_VENDOR = 10
 DRIVER_TYPE_FOOMATIC_RECOMMENDED_POSTSCRIPT = 15
+DRIVER_TYPE_HPCUPS = 16
 DRIVER_TYPE_FOOMATIC_HPIJS_ON_HP = 17
 DRIVER_TYPE_GUTENPRINT_NATIVE_SIMPLIFIED = 20
 DRIVER_TYPE_GUTENPRINT_NATIVE = 25
 DRIVER_TYPE_SPLIX = 27
 DRIVER_TYPE_FOOMATIC_PS = 30
 DRIVER_TYPE_FOOMATIC_HPIJS = 40
-DRIVER_TYPE_HPCUPS = 45
 DRIVER_TYPE_FOOMATIC_GUTENPRINT_SIMPLIFIED = 50
 DRIVER_TYPE_FOOMATIC_GUTENPRINT = 60
 DRIVER_TYPE_FOOMATIC = 70
@@ -698,16 +706,18 @@ class PPDs:
         if not id_matched:
             sanitised_uri = re.sub (pattern="//[^@]*@/?", repl="//",
                                     string=str (uri))
-            print "No ID match for device %s:" % sanitised_uri
-            print "  <manufacturer>%s</manufacturer>" % mfg
-            print "  <model>%s</model>" % mdl
-            print "  <description>%s</description>" % description
             try:
                 cmd = reduce (lambda x, y: x + ","+ y, commandsets)
             except TypeError:
                 cmd = ""
+            id = "MFG:%s;MDL:%s;" % (mfg, mdl)
+            if cmd:
+                id += "CMD:%s;" % cmd
+            if description:
+                id += "DES:%s;" % description
 
-            print "  <commandset>%s</commandset>" % cmd
+            print "No ID match for device %s:" % sanitised_uri
+            print id
 
         print "Using %s (status: %d)" % (ppdnamelist[0], status)
         return (status, ppdnamelist[0])
@@ -855,15 +865,15 @@ class PPDs:
             return get ("PostScript Printer")
         elif (("pclxl" in cmdsets) or ("pcl-xl" in cmdsets) or
               ("pcl6" in cmdsets) or ("pcl 6 emulation" in cmdsets)):
-            return get ("PCL 6/PCL XL Printer")
+            return get ("PCL 6/PCL XL Printer", "PCL Laser Printer")
         elif "pcl5e" in cmdsets:
-            return get ("PCL 5e Printer")
+            return get ("PCL 5e Printer", "PCL Laser Printer")
         elif "pcl5c" in cmdsets:
-            return get ("PCL 5c Printer")
+            return get ("PCL 5c Printer", "PCL Laser Printer")
         elif ("pcl5" in cmdsets) or ("pcl 5 emulation" in cmdsets):
-            return get ("PCL 5 Printer")
+            return get ("PCL 5 Printer", "PCL Laser Printer")
         elif "pcl" in cmdsets:
-            return get ("PCL 3 Printer")
+            return get ("PCL 3 Printer", "PCL Laser Printer")
         elif (("escpl2" in cmdsets) or ("esc/p2" in cmdsets) or
               ("escp2e" in cmdsets)):
             return get ("ESC/P Dot Matrix Printer")
@@ -1036,7 +1046,7 @@ def _self_test(argv):
          "DES:Canon iP3000;VER:1.09;STA:10;FSI:03;", 1, "Canon PIXMA iP3000"),
         ("MFG:HP;MDL:Deskjet 5400 series;CMD:MLC,PCL,PML,DW-PCL,DESKJET,DYN;"
          "1284.4DL:4d,4e,1;CLS:PRINTER;DES:5440;",
-         1, "HP DeskJet (5440)|(5550)"), # foomatic-db-hpijs used to say 5440
+         1, "HP DeskJet (5440|5550)"), # foomatic-db-hpijs used to say 5440
         ("MFG:Hewlett-Packard;MDL:HP LaserJet 3390;"
          "CMD:PJL,MLC,PCL,POSTSCRIPT,PCLXL;",
          0, "HP LaserJet 3390"),
@@ -1049,7 +1059,6 @@ def _self_test(argv):
         ("MFG:New;MDL:Unknown PCL5c Printer;CMD:PCL5c;", 2, "Generic PCL 5c"),
         ("MFG:New;MDL:Unknown PCL5 Printer;CMD:PCL5;", 2, "Generic PCL 5"),
         ("MFG:New;MDL:Unknown PCL3 Printer;CMD:PCL;", 2, "Generic PCL"),
-        ("MFG:New;MDL:Unknown ESC/P Printer;CMD:ESCP2E;", 2, "Generic ESC/P"),
         ("MFG:New;MDL:Unknown Printer;", 100, None),
         ]
 
