@@ -40,6 +40,7 @@ import subprocess
 import sys
 import time
 import urllib
+from xml.sax import saxutils
 
 from debug import *
 import config
@@ -61,13 +62,14 @@ errordialogs.set_gettext_function (_)
 
 pkgdata = config.pkgdatadir
 ICON="printer"
+ICON_SIZE=22
 SEARCHING_ICON="document-print-preview"
 
 # We need to call pynotify.init before we can check the server for caps
 pynotify.init('System Config Printer Notification')
 
 class PrinterURIIndex:
-    def __init__ (self, names=None):
+    def __init__ (self, names=[]):
         self.printer = {}
         self.names = names
         c = cups.Connection ()
@@ -136,14 +138,15 @@ class PrinterURIIndex:
 
         name = attrs['printer-name']
         self.update_from_attrs (name, attrs)
-        self.printer[uri] = name
-        return self.printer[uri]
+        if uri != None:
+            self.printer[uri] = name
+        return name
 
 
 class CancelJobsOperation:
     def __init__ (self, jobviewer, jobids, purge_job):
         self.jobviewer = jobviewer
-        self.jobids = jobids
+        self.jobids = list (jobids)
         self.purge_job = purge_job
         if purge_job:
             if len(self.jobids) > 1:
@@ -164,8 +167,8 @@ class CancelJobsOperation:
                              gtk.DIALOG_MODAL |
                              gtk.DIALOG_DESTROY_WITH_PARENT |
                              gtk.DIALOG_NO_SEPARATOR,
-                             (gtk.STOCK_NO, gtk.RESPONSE_NO,
-                              gtk.STOCK_YES, gtk.RESPONSE_YES))
+                             (_("Keep Printing"), gtk.RESPONSE_NO,
+                              dialog_title, gtk.RESPONSE_YES))
         dialog.set_default_response (gtk.RESPONSE_NO)
         dialog.set_border_width (6)
         dialog.set_resizable (False)
@@ -209,10 +212,17 @@ class CancelJobsOperation:
         if response != gtk.RESPONSE_YES:
             return
 
-        c = asyncconn.Connection (host=self.jobviewer.host,
-                                  port=self.jobviewer.port,
-                                  encryption=self.jobviewer.encryption)
-        self.connection = c
+        asyncconn.Connection (host=self.jobviewer.host,
+                              port=self.jobviewer.port,
+                              encryption=self.jobviewer.encryption,
+                              reply_handler=self._connected,
+                              error_handler=self._connect_failed)
+
+    def _connect_failed (self, connection, exc):
+        debugprint ("CancelJobsOperation._connect_failed %s:%s" % (connection, repr (exc)))
+
+    def _connected (self, connection, result):
+        self.connection = connection
 
         if self.purge_job:
             operation = _("deleting job")
@@ -229,17 +239,17 @@ class CancelJobsOperation:
         if self.jobviewer == None:
             return
 
-        self.connection._end_operation ()
-        self.jobviewer.update_monitor ()
         if type (exc) == cups.IPPError:
             (e, m) = exc.args
             if (e != cups.IPP_NOT_POSSIBLE and
                 e != cups.IPP_NOT_FOUND):
                 self.jobviewer.show_IPP_Error (e, m)
-
-            return
-
-        raise exc
+            self.cancelJob_finish(connection, None)
+        else:
+            self.connection._end_operation ()
+            self.connection.destroy ()
+            self.jobviewer.update_monitor ()
+            raise exc
 
     def cancelJob_finish (self, connection, result):
         debugprint ("cancelJob_finish %s:%s" % (connection, repr (result)))
@@ -247,15 +257,16 @@ class CancelJobsOperation:
             return
 
         del self.jobids[0]
-        try:
+        if not self.jobids:
+            # Last job canceled.
+            self.connection._end_operation ()
+            self.connection.destroy ()
+            self.jobviewer.update_monitor ()
+        else:
+            # there are other jobs to cancel/delete
             connection.cancelJob (self.jobids[0], self.purge_job,
                                   reply_handler=self.cancelJob_finish,
                                   error_handler=self.cancelJob_error)
-        except IndexError:
-            # Last job canceled.
-            self.connection._end_operation ()
-            self.jobviewer.update_monitor ()
-            return
 
 class JobViewer (GtkGUI, monitor.Watcher):
     required_job_attributes = set(['job-k-octets',
@@ -296,9 +307,9 @@ class JobViewer (GtkGUI, monitor.Watcher):
 
         self.getWidgets ({"JobsWindow":
                               ["JobsWindow",
-                               "job_menubar_item",
                                "treeview",
-                               "statusbar"],
+                               "statusbar",
+                               "toolbar"],
                           "statusicon_popupmenu":
                               ["statusicon_popupmenu"]},
 
@@ -306,23 +317,25 @@ class JobViewer (GtkGUI, monitor.Watcher):
 
         job_action_group = gtk.ActionGroup ("JobActionGroup")
         job_action_group.add_actions ([
-                ("cancel-job", gtk.STOCK_CANCEL, _("_Cancel"), None, None,
-                 self.on_job_cancel_activate),
-                ("delete-job", gtk.STOCK_DELETE, _("_Delete"), None, None,
-                 self.on_job_delete_activate),
-                ("hold-job", gtk.STOCK_MEDIA_PAUSE, _("_Hold"), None, None,
-                 self.on_job_hold_activate),
-                ("release-job", gtk.STOCK_MEDIA_PLAY, _("_Release"), None, None,
-                 self.on_job_release_activate),
-                ("reprint-job", gtk.STOCK_REDO, _("Re_print"), None, None,
-                 self.on_job_reprint_activate),
-                ("retrieve-job", gtk.STOCK_SAVE_AS, _("Re_trieve"), None, None,
-                 self.on_job_retrieve_activate),
+                ("cancel-job", gtk.STOCK_CANCEL, _("_Cancel"), None,
+                 _("Cancel selected jobs"), self.on_job_cancel_activate),
+                ("delete-job", gtk.STOCK_DELETE, _("_Delete"), None,
+                 _("Delete selected jobs"), self.on_job_delete_activate),
+                ("hold-job", gtk.STOCK_MEDIA_PAUSE, _("_Hold"), None,
+                 _("Hold selected jobs"), self.on_job_hold_activate),
+                ("release-job", gtk.STOCK_MEDIA_PLAY, _("_Release"), None,
+                 _("Release selected jobs"), self.on_job_release_activate),
+                ("reprint-job", gtk.STOCK_REDO, _("Re_print"), None,
+                 _("Reprint selected jobs"), self.on_job_reprint_activate),
+                ("retrieve-job", gtk.STOCK_SAVE_AS, _("Re_trieve"), None,
+                 _("Retrieve selected jobs"), self.on_job_retrieve_activate),
                 ("move-job", None, _("_Move To"), None, None, None),
                 ("authenticate-job", None, _("_Authenticate"), None, None,
                  self.on_job_authenticate_activate),
-                 ("job-attributes", None, _("_View Attributes"), None, None,
-                 self.on_job_attributes_activate)
+                ("job-attributes", None, _("_View Attributes"), None, None,
+                 self.on_job_attributes_activate),
+                ("close", gtk.STOCK_CLOSE, None, "<ctrl>w",
+                 _("Close this window"), self.on_delete_event)
                 ])
         self.job_ui_manager = gtk.UIManager ()
         self.job_ui_manager.insert_action_group (job_action_group, -1)
@@ -338,6 +351,7 @@ class JobViewer (GtkGUI, monitor.Watcher):
  <accelerator action="move-job"/>
  <accelerator action="authenticate-job"/>
  <accelerator action="job-attributes"/>
+ <accelerator action="close"/>
 </ui>
 """
 )
@@ -369,7 +383,19 @@ class JobViewer (GtkGUI, monitor.Watcher):
             item.show ()
             self.job_context_menu.append (item)
 
-        self.job_menubar_item.set_submenu (self.job_context_menu)
+        for action_name in ["cancel-job",
+                            "delete-job",
+                            "hold-job",
+                            "release-job",
+                            "reprint-job",
+                            "retrieve-job",
+                            "close"]:
+            action = job_action_group.get_action (action_name)
+            action.set_sensitive (action_name == "close")
+            action.set_is_important (action_name == "close")
+            item = action.create_tool_item ()
+            item.show ()
+            self.toolbar.insert (item, -1)
 
         for skip, ellipsize, name, setter in \
                 [(False, False, _("Job"), self._set_job_job_number_text),
@@ -443,10 +469,20 @@ class JobViewer (GtkGUI, monitor.Watcher):
 
         self.statusbar_set = False
 
+        def load_icon(theme, icon):
+            try:
+                pixbuf = theme.load_icon (icon, ICON_SIZE, 0)
+            except gobject.GError:
+                debugprint ("No %s icon available" % icon)
+                # Just create an empty pixbuf.
+                pixbuf = gtk.gdk.Pixbuf (gtk.gdk.COLORSPACE_RGB,
+                                         True, 8, ICON_SIZE, ICON_SIZE)
+                pixbuf.fill (0)
+            return pixbuf
+
         theme = gtk.icon_theme_get_default ()
-        self.icon_jobs = theme.load_icon (ICON, 22, 0)
-        self.icon_jobs_processing = theme.load_icon ("printer-printing",
-                                                     22, 0)
+        self.icon_jobs = load_icon (theme, ICON)
+        self.icon_jobs_processing = load_icon (theme, "printer-printing")
         self.icon_no_jobs = self.icon_jobs.copy ()
         self.icon_no_jobs.fill (0)
         self.icon_jobs.composite (self.icon_no_jobs,
@@ -459,8 +495,8 @@ class JobViewer (GtkGUI, monitor.Watcher):
                                   127)
         if self.trayicon:
             self.statusicon = gtk.StatusIcon ()
-            pixbuf = theme.load_icon (ICON, 22, 0)
-            self.statusicon.set_from_pixbuf (pixbuf)    
+            pixbuf = load_icon (theme, ICON)
+            self.statusicon.set_from_pixbuf (pixbuf)
             self.set_statusicon_from_pixbuf (self.icon_no_jobs)
             self.statusicon.connect ('activate', self.toggle_window_display)
             self.statusicon.connect ('popup-menu', self.on_icon_popupmenu)
@@ -488,8 +524,33 @@ class JobViewer (GtkGUI, monitor.Watcher):
         self.JobsAttributesWindow.set_transient_for (self.JobsWindow)
         self.JobsAttributesWindow.connect("delete_event",
                                           self.job_attributes_on_delete_event)
+        self.JobsAttributesWindow.add_accel_group (self.job_ui_manager.get_accel_group ())
+        attrs_action_group = gtk.ActionGroup ("AttrsActionGroup")
+        attrs_action_group.add_actions ([
+                ("close", gtk.STOCK_CLOSE, None, "<ctrl>w",
+                 _("Close this window"), self.job_attributes_on_delete_event)
+                ])
+        self.attrs_ui_manager = gtk.UIManager ()
+        self.attrs_ui_manager.insert_action_group (attrs_action_group, -1)
+        self.attrs_ui_manager.add_ui_from_string (
+"""
+<ui>
+ <accelerator action="close"/>
+</ui>
+"""
+)
+        self.attrs_ui_manager.ensure_update ()
+        self.JobsAttributesWindow.add_accel_group (self.attrs_ui_manager.get_accel_group ())
+        vbox = gtk.VBox ()
+        self.JobsAttributesWindow.add (vbox)
+        toolbar = gtk.Toolbar ()
+        action = self.attrs_ui_manager.get_action ("/close")
+        item = action.create_tool_item ()
+        item.set_is_important (True)
+        toolbar.insert (item, 0)
+        vbox.pack_start (toolbar, False, False, 0)
         self.notebook = gtk.Notebook()
-        self.JobsAttributesWindow.add(self.notebook)
+        vbox.pack_start (self.notebook)
 
     def cleanup (self):
         self.monitor.cleanup ()
@@ -566,7 +627,7 @@ class JobViewer (GtkGUI, monitor.Watcher):
             self.loop.quit ()
         return True
 
-    def job_attributes_on_delete_event(self, widget, event):
+    def job_attributes_on_delete_event(self, widget, event=None):
         for page in range(self.notebook.get_n_pages()):
             self.notebook.remove_page(-1)
         self.jobs_attrs = {}
@@ -603,8 +664,8 @@ class JobViewer (GtkGUI, monitor.Watcher):
 
         self.JobsWindow.set_data ('visible', not visible)
 
-    def on_show_completed_jobs_activate(self, menuitem):
-        if menuitem.get_active():
+    def on_show_completed_jobs_clicked(self, toggletoolbutton):
+        if toggletoolbutton.get_active():
             which_jobs = "all"
         else:
             which_jobs = "not-completed"
@@ -615,9 +676,6 @@ class JobViewer (GtkGUI, monitor.Watcher):
         now = time.time ()
         need_update = False
         for job, data in self.jobs.iteritems():
-            if self.jobs.has_key (job):
-                iter = self.jobiters[job]
-
             t = _("Unknown")
             if data.has_key ('time-at-creation'):
                 created = data['time-at-creation']
@@ -650,10 +708,18 @@ class JobViewer (GtkGUI, monitor.Watcher):
                     need_update = False
                     t = time.strftime ("%B %Y", time.localtime (created))
 
-            self.store.set_value (iter, 1, t)
+            if self.jobiters.has_key (job):
+                iter = self.jobiters[job]
+                self.store.set_value (iter, 1, t)
 
         if need_update and not self.job_creation_times_timer:
-            t = gobject.timeout_add_seconds (60, self.update_job_creation_times)
+            def update_times_with_locking ():
+                gtk.gdk.threads_enter ()
+                ret = self.update_job_creation_times ()
+                gtk.gdk.threads_leave ()
+                return ret
+
+            t = gobject.timeout_add_seconds (60, update_times_with_locking)
             self.job_creation_times_timer = t
 
         if not need_update:
@@ -699,7 +765,9 @@ class JobViewer (GtkGUI, monitor.Watcher):
 
         if not self.job_creation_times_timer:
             def start_updating_job_creation_times():
+                gtk.gdk.threads_enter ()
                 self.update_job_creation_times ()
+                gtk.gdk.threads_leave ()
                 return False
 
             gobject.timeout_add (500, start_updating_job_creation_times)
@@ -731,6 +799,9 @@ class JobViewer (GtkGUI, monitor.Watcher):
                 pass
             except AttributeError:
                 pass
+            except cups.IPPError:
+                # someone else may have purged the job
+                return
 
             if attrs:
                 data.update (attrs)
@@ -869,6 +940,7 @@ class JobViewer (GtkGUI, monitor.Watcher):
                 username = pwd.getpwuid (os.getuid ())[0]
                 keyring_attrs["user"] = str (username)
                 self.display_auth_info_dialog (job, keyring_attrs)
+        self.update_sensitivity ()
 
     def display_auth_info_dialog (self, job, keyring_attrs=None):
         data = self.jobs[job]
@@ -1005,8 +1077,9 @@ class JobViewer (GtkGUI, monitor.Watcher):
         if event.button == 3:
             self.show_treeview_popup_menu (treeview, event, event.button)
 
-    def on_selection_changed (self, selection):
-        treeview = selection.get_tree_view()
+    def update_sensitivity (self, selection = None):
+        if (selection is None):
+            selection = self.treeview.get_selection () 
         (model, pathlist) = selection.get_selected_rows()
         cancel = self.job_ui_manager.get_action ("/cancel-job")
         delete = self.job_ui_manager.get_action ("/delete-job")
@@ -1057,7 +1130,10 @@ class JobViewer (GtkGUI, monitor.Watcher):
 
             uri = job.get ('job-printer-uri', None)
             if uri:
-                printer = self.printer_uri_index.lookup (uri)
+                try:
+                    printer = self.printer_uri_index.lookup (uri)
+                except KeyError:
+                    printer = uri
                 job_printers[printer] = uri
 
         if len (job_printers.keys ()) == 1:
@@ -1091,6 +1167,9 @@ class JobViewer (GtkGUI, monitor.Watcher):
         authenticate.set_sensitive(authenticate_sensitive)
         attributes.set_sensitive(True)
 
+    def on_selection_changed (self, selection):
+        self.update_sensitivity (selection)
+
     def show_treeview_popup_menu (self, treeview, event, event_button):
         # Right-clicked.
         self.job_context_menu.popup (None, None, None, event_button,
@@ -1129,8 +1208,9 @@ class JobViewer (GtkGUI, monitor.Watcher):
         self.on_job_cancel_activate2(True)
 
     def on_job_cancel_activate2(self, purge_job):
-        op = CancelJobsOperation (self, self.jobids, purge_job)
-        self.ops.append (op)
+        if self.jobids:
+            op = CancelJobsOperation (self, self.jobids, purge_job)
+            self.ops.append (op)
 
     def on_job_hold_activate(self, menuitem):
         try:
@@ -1290,7 +1370,7 @@ class JobViewer (GtkGUI, monitor.Watcher):
         for jobid in self.jobids:
             self.display_auth_info_dialog (jobid)
 
-    def on_refresh_activate(self, menuitem):
+    def on_refresh_clicked(self, toolbutton):
         self.monitor.refresh ()
         self.update_job_creation_times ()
 
@@ -1507,6 +1587,20 @@ class JobViewer (GtkGUI, monitor.Watcher):
             # Not important enough to justify a notification.
             return
 
+        blacklist = [
+            # Some printers report 'other-warning' for no apparent
+            # reason, e.g.  Canon iR 3170C, Epson AL-CX11NF.
+            # See bug #520815.
+            "other",
+
+            # This seems to be some sort of 'magic' state reason that
+            # is for internal use only.
+            "com.apple.print.recoverable",
+            ]
+
+        if reason.get_reason () in blacklist:
+            return
+
         self.notify_printer_state_reason (reason)
 
     def notify_printer_state_reason (self, reason):
@@ -1515,9 +1609,6 @@ class JobViewer (GtkGUI, monitor.Watcher):
             debugprint ("Already sent notification for %s" % repr (reason))
             return
 
-        if reason.get_reason () == "com.apple.print.recoverable":
-            return
-
         level = reason.get_level ()
         if (level == StateReason.ERROR or
             reason.get_reason () == "connecting-to-device"):
@@ -1566,8 +1657,12 @@ class JobViewer (GtkGUI, monitor.Watcher):
                                          host=self.host,
                                          port=self.port,
                                          encryption=self.encryption)
-                attrs = c.getPrinterAttributes (uri=printer_uri,
-                                                requested_attributes=pattrs)
+                try:
+                    attrs = c.getPrinterAttributes (uri=printer_uri,
+                                                    requested_attributes=pattrs)
+                except cups.IPPError:
+                    return
+
                 device_uri = attrs.get ('device-uri')
 
             if device_uri != None:
@@ -1625,7 +1720,8 @@ class JobViewer (GtkGUI, monitor.Watcher):
         self.jobs = jobs
         self.active_jobs = set()
         for jobid, jobdata in jobs.iteritems ():
-            if self.job_is_active (jobdata):
+            if (self.job_is_active (jobdata) and
+                jobdata.has_key('job-printer-name')):
                 self.active_jobs.add (jobid)
 
         self.set_process_pending (True)
@@ -1779,7 +1875,7 @@ class JobViewer (GtkGUI, monitor.Watcher):
 
                 markup = ('<span weight="bold" size="larger">' +
                           _("Print Error") + '</span>\n\n' +
-                          message)
+                          saxutils.escape (message))
                 try:
                     if event['printer-state'] == cups.IPP_PRINTER_STOPPED:
                         name = event['printer-name']
