@@ -28,6 +28,7 @@ import subprocess
 import threading
 import errno
 import cups
+import gobject
 
 try:
     import pysmb
@@ -191,7 +192,9 @@ class BackgroundSmbAuthContext(pysmb.AuthContext):
         pysmb.AuthContext.__init__ (self, *args, **kwargs)
 
     def _do_perform_authentication (self):
+        gtk.gdk.threads_enter ()
         result = pysmb.AuthContext.perform_authentication (self)
+        gtk.gdk.threads_leave ()
         self._do_perform_authentication_result = result
         self._gui_event.set ()
         
@@ -257,6 +260,7 @@ class PrinterFinder:
         try:
             p = subprocess.Popen (args=["/usr/lib/cups/backend/snmp",
                                         self.hostname],
+                                  close_fds=True,
                                   stdin=null,
                                   stdout=subprocess.PIPE,
                                   stderr=null)
@@ -300,6 +304,7 @@ class PrinterFinder:
             # Cache the make and model for use by other search methods
             # that are not able to determine it.
             self._cached_attributes['device-make-and-model'] = make_and_model
+            self._cached_attributes['device_id'] = device_id
 
     def _probe_lpd (self):
         lpd = LpdServer (self.hostname)
@@ -321,6 +326,7 @@ class PrinterFinder:
         null = file ("/dev/null", "r+")
         try:
             p = subprocess.Popen (args=["hp-makeuri", "-c", self.hostname],
+                                  close_fds=True,
                                   stdin=null,
                                   stdout=subprocess.PIPE,
                                   stderr=null)
